<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Helpers_Reviews extends Shopgate_Model_Review
{
    /**
     * returns a Shopgate review title from review text
     *
     * @param string $text
     *
     * @return string
     */
    public function buildTitle($text)
    {
        return substr($text, 0, 20) . "...";
    }
    
    /**
     * returns a Shopgate time string
     *
     * @param string $date
     *
     * @return string
     */
    public function buildDate($date)
    {
        return empty($date) ? "" : strftime("%Y-%m-%d", strtotime($date));
    }
    
    /**
     * calculates shopgate score from shop score
     *
     * @param int $shopScore
     *
     * @return int
     */
    public function buildScore($shopScore)
    {
        return intval($shopScore * 2);
    }
    
    /**
     * generate a query to read the review data from the database
     * 
     * @param int|null   $limit
     * @param int|null   $offset
     * @param array|null $uids
     *
     * @return string
     */
    public function buildReviewQuery($limit = null, $offset = null, $uids = null)
    {
        $sqlQuery = "SELECT\n" . "\ttbl_r.reviews_id,\n" . "\ttbl_r.products_id,\n"
            . "\ttbl_r.customers_name,\n" . "\ttbl_r.reviews_rating,\n"
            . "\ttbl_r.date_added,\n" . "\ttbl_rd.reviews_text\n" . "FROM "
            . TABLE_REVIEWS . " AS tbl_r\n" . "\tLEFT JOIN "
            . TABLE_REVIEWS_DESCRIPTION
            . " AS tbl_rd ON (tbl_r.reviews_id=tbl_rd.reviews_id)\n" . "WHERE\n"
            . // OsCommerce-version dependent! (field "reviews_status" only available in osCommerce 2.3+)
            "\t" . ((defined('PROJECT_VERSION') && strpos(PROJECT_VERSION, '2.3') !== false)
                ? "tbl_r.reviews_status=1" : "1=1")
            . "\n" . (!empty($uids) ? "AND\n\ttbl_r.reviews_id in (" . implode(',', $uids) . ")"
                . "\n" : "") . "ORDER BY\n" . "\ttbl_r.products_id\n"
            . ((is_numeric($limit) && is_numeric($offset)) ? "LIMIT " . $offset . "," . $limit : "") . ";";
        
        return $sqlQuery;
    }
}
