<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Helpers_Shipping
{
    /** @var Shopgate_Helpers_Tax */
    protected $taxHelper;
    /** @var Shopgate_Helper_Pricing */
    protected $pricingHelper;
    /** @var ShopgateShippingMethod */
    protected $shippingMethod;

    /**
     * @param Shopgate_Helpers_Tax    $taxHelper
     * @param Shopgate_Helper_Pricing $pricingHelper
     * @param ShopgateShippingMethod  $shippingMethod
     */
    public function __construct(
        Shopgate_Helpers_Tax $taxHelper,
        Shopgate_Helper_Pricing $pricingHelper,
        ShopgateShippingMethod $shippingMethod
    ) {
        $this->taxHelper      = $taxHelper;
        $this->pricingHelper  = $pricingHelper;
        $this->shippingMethod = $shippingMethod;
    }

    /**
     * Parses the passed shipping modules
     * and pulls the shipping methods
     * into exportable methods
     *
     * @param array $shippingModules
     *
     * @return ShopgateShippingMethod[]
     */
    public function getShippingMethodExport(array $shippingModules)
    {
        $exportMethods = array();
        foreach ($shippingModules as $module) {
            //we don`t support usps as shopgate plugin shipping method
            //also on error continue
            if (strpos($module['module'], 'United States Postal Service') !== false
                || !empty($module['error'])
                || !is_array($module['methods'])
            ) {
                continue;
            }

            foreach ($module['methods'] as $method) {
                /** @var ShopgateShippingMethod $exportMethod */
                $exportMethod = new $this->shippingMethod;
                $cost         = $method['cost'];
                $exportMethod->setId(empty($method['id']) ? $module['id'] : $method['id']);
                $exportMethod->setTitle(empty($method['title']) ? $module['module'] : $method['title']);

                if (!empty($module['tax'])) {
                    $exportMethod->setTaxClass($this->taxHelper->getTaxClassByValue($module['tax']));
                    $exportMethod->setTaxPercent($module['tax']);
                    $costWithTax =
                        $this->pricingHelper->formatPriceNumber($cost * (1 + ($module['tax'] / 100)), 2);
                    $exportMethod->setAmountWithTax($costWithTax);
                    $exportMethod->setAmount($cost);
                } else {
                    $exportMethod->setTaxPercent(0);
                    $exportMethod->setAmountWithTax($cost);
                    $exportMethod->setAmount($cost);
                }

                $exportMethods[] = $exportMethod;
            }
        }

        return $exportMethods;
    }
}
