<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

/**
 * Manipulates shopgate order beyond the regular functionality
 */
class Shopgate_Helpers_ShopgateOrder
{
    /** @var ShopgateOrder */
    protected $shopgateOrder;
    /** @var Shopgate_Models_Products_Native */
    protected $productModel;

    /**
     * @param ShopgateOrder                   $shopgateOrder
     * @param Shopgate_Models_Products_Native $productModel
     */
    public function __construct(ShopgateOrder $shopgateOrder, Shopgate_Models_Products_Native $productModel)
    {
        $this->shopgateOrder = $shopgateOrder;
        $this->productModel  = $productModel;
    }

    /**
     * Checks whether cart items exist in the system,
     * throws an exception if they do not.
     *
     * @return bool
     * @throws ShopgateLibraryException
     */
    public function validateItemsExist()
    {
        $itemIds     = $this->getItemIds();
        $implodedIds = implode(',', $itemIds);
        $this->productModel->getSelect()
                           ->where('products_id IN (' . $implodedIds . ')');
        $this->productModel->getCollection();

        if ($this->productModel->getSize() !== count($itemIds)) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_ORDER_ITEM_NOT_FOUND,
                "Collection ids expected: '{$implodedIds}' returned: '{$this->productModel->getAllIds()}'", true, true
            );
        }

        return true;
    }

    /**
     * Gets all item id's in the cart. See coupon skip flag.
     *
     * @param bool $removeCoupons - whether to remove coupon items & payment fees
     *
     * @return int[]
     */
    public function getItemIds($removeCoupons = true)
    {
        $itemIds = array();

        /**
         * @var ShopgateOrderItem $item
         */
        foreach ($this->shopgateOrder->getItems() as $item) {
            if ($removeCoupons && ($item->isSgCoupon() || $item->isPayment())) {
                continue;
            }
            $itemIds[] = (int)$item->getItemNumber();
        }

        return array_unique($itemIds);
    }
}
