<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
class Shopgate_Helpers_Zones
{
    /** @var Shopgate_Models_Countries_Native */
    protected $countryModel;
    /** @var Shopgate_Models_Zones_Native */
    protected $zoneModel;

    /**
     * @param Shopgate_Models_Countries_Native $countryModel
     * @param Shopgate_Models_Zones_Native     $zoneModel
     */
    public function __construct(Shopgate_Models_Countries_Native $countryModel, Shopgate_Models_Zones_Native $zoneModel)
    {
        $this->countryModel = $countryModel;
        $this->zoneModel    = $zoneModel;
    }

    /**
     * Read the country id from the database by iso-2 code
     *
     * @param string $name
     *
     * @return mixed
     * @deprecated
     */
    public function getCountryIdByName($name)
    {
        $query         =
            "SELECT c.countries_id AS id FROM " . TABLE_COUNTRIES . " AS c WHERE c.countries_iso_code_2 = \"{$name}\"";
        $result        = ShopgateWrapper::db_query($query);
        $countryResult = ShopgateWrapper::db_fetch_array($result);

        return $countryResult['id'];
    }

    /**
     * Retrieves country object from database by iso-2 code
     *
     * @param string $name
     *
     * @return Shopgate_Models_Countries_Native
     */
    public function getCountryByIso2Name($name)
    {
        $this->countryModel->load($name, 'countries_iso_code_2');

        return $this->countryModel;
    }

    /**
     * read the zone data from database by the zone country id
     *
     * @param string | int $zoneCountryId
     *
     * @return array
     */
    public function getZoneByCountryId($zoneCountryId)
    {
        $query         = "SELECT * FROM " . TABLE_ZONES_TO_GEO_ZONES . " WHERE zone_country_id = '" . $zoneCountryId
            . "' ORDER BY zone_id";
        $result        = ShopgateWrapper::db_query($query);
        $countryResult = ShopgateWrapper::db_fetch_array($result);

        return $countryResult;
    }

    /**
     * reads the country data for the shipping module "flat" from the database
     *
     * @param string | int $zoneCountryId
     *
     * @return array
     */
    public function getZoneByCountryIdModuleShippingFlat($zoneCountryId)
    {
        $query         =
            "SELECT * FROM " . TABLE_ZONES_TO_GEO_ZONES . " WHERE geo_zone_id = '" . MODULE_SHIPPING_FLAT_ZONE
            . "' AND zone_country_id = '" . $zoneCountryId . "' ORDER BY zone_id";
        $result        = ShopgateWrapper::db_query($query);
        $countryResult = ShopgateWrapper::db_fetch_array($result);

        return $countryResult;
    }

    /**
     * read the zone id from database by the zone code and country id
     *
     * @param string | int $countryId
     * @param string       $zoneCode
     *
     * @return mixed
     */
    public function getCustomerZoneId($countryId, $zoneCode)
    {
        $query      =
            "select z.zone_id from " . TABLE_ZONES
            . " as z where z.zone_country_id = {$countryId} AND z.zone_code = '{$zoneCode}'";
        $result     = ShopgateWrapper::db_query($query);
        $zoneResult = ShopgateWrapper::db_fetch_array($result);

        return $zoneResult['zone_id'];
    }
}
