<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

/**
 * Main class to manipulate collections
 */
class Shopgate_Models_Abstract_Db_Collection extends Shopgate_Models_Abstract_Db_Item implements
    Shopgate_Interfaces_Db_ItemInterface,
    Shopgate_Interfaces_Db_CollectionInterface,
    Iterator
{
    /**
     * Retrieves the DbItem collection
     *
     * @var Shopgate_Models_Abstract_Db_Item[]
     */
    protected $collection = array();

    /**
     * Collection setter
     *
     * @param Shopgate_Models_Abstract_Db_Item[] $collection
     *
     * @return $this
     */
    private function setCollection($collection)
    {
        $this->collection = $collection;

        return $this;
    }

    /**
     * Collection getter based on Main table
     *
     * @return $this
     */
    public function getCollection()
    {
        if (!$this->collection) {
            $this->prepareSelectFrom();
            $collection = $this->tableToObject($this->fetchSelectResults());
            $this->setCollection($collection);
        }

        return $this;
    }

    /**
     * Attempts to retrieve the first object of the collection
     *
     * @return $this|Shopgate_Models_Abstract_Db_Item
     */
    public function getFirstItem()
    {
        if (!isset($this->collection[0])) {
            return $this;
        }

        return $this->collection[0];
    }

    /**
     * Helper that allows to transfer table array row
     * into a collection of objects
     *
     * @param array $tableRows
     *
     * @return Shopgate_Models_Abstract_Db_Item[]
     */
    private function tableToObject($tableRows)
    {
        $list = array();
        foreach ($tableRows as $row) {
            /** @var Shopgate_Models_Abstract_Db_Item $item */
            $item = new $this();
            $item->setMainTable($this->getMainTable());
            $item->setData($row);
            $this->setOrigData();
            $item->setObjectToOld();
            $list[] = $item;
        }

        return $list;
    }

    /**
     * Only saves objects that were modified
     * inside the collection or saves object
     * if the collection was not loaded
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        if ($this->getSize() > 0) {
            foreach ($this->getCollection() as $item) {
                $item->save();
            }
        } else {
            parent::save();
        }

        return $this;
    }

    /**
     * Load only works if a collection was not loaded. If we don't,
     * we will have issues with the save method as it will prioritize
     * collection saving over object data saving
     *
     * @param int    $index  - table primary key ID
     * @param string $column - name of the primary unique table
     *
     * @return $this
     * @throws Exception
     */
    public function load($index, $column = '')
    {
        if ($this->getSize() > 0) {
            throw $this->getDbException('Cannot load object if collection was loaded previously');
        } else {
            parent::load($index, $column);
        }

        return $this;
    }

    /**
     * Outputs size of the collection
     *
     * @return int
     */
    public function getSize()
    {
        return count($this->collection);
    }

    /**
     * Returns a string of item ids in the collection
     *
     * @return string
     */
    public function getAllIds()
    {
        $ids = array();
        foreach ($this->collection as $item) {
            $ids[] = $item->getId();
        }

        return implode(',', $ids);
    }

    /** =========   Iterator Functions  =============== */

    /**
     * Rewind function rewrite
     */
    public function rewind()
    {
        reset($this->collection);
    }

    /**
     * Current pointer rewrite
     *
     * @return mixed
     */
    public function current()
    {
        return current($this->collection);
    }

    /**
     * Next pointer rewrite
     *
     * @return mixed
     */
    public function next()
    {
        return next($this->collection);
    }

    /**
     * Key pointer rewrite
     *
     * @return mixed
     */
    public function key()
    {
        return key($this->collection);
    }

    /**
     * Validation check rewrite
     *
     * @return bool
     */
    public function valid()
    {
        $key = $this->key();

        return ($key !== null && $key !== false);
    }
}
