<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

/**
 * Shopgate customer table class
 *
 * @method $this setCustomerId($id_int)
 * @method int getCustomerId()
 * @method $this setCustomerToken($token_str)
 * @method string getCustomerToken()
 */
class Shopgate_Models_Customers_Shopgate extends Shopgate_Models_Abstract_Db_Collection
{
    protected $tableName   = TABLE_CUSTOMERS_SHOPGATE_CUSTOMER;
    protected $idFieldName = 'customer_id';
    /** @var Shopgate_Helpers_Zones */
    protected $zoneHelper;
    /** @var Shopgate_Models_Customers_Native */
    protected $nativeCustomerModel;

    /**
     * @inheritdoc
     */
    public function __construct(
        Shopgate_Helpers_Zones $zoneHelper,
        Shopgate_Models_Customers_Native $customerModel
    ) {
        $this->zoneHelper          = $zoneHelper;
        $this->nativeCustomerModel = $customerModel;
        parent::__construct();
    }

    /**
     * Add zone and country uid to existing cart address
     *
     * @param ShopgateCart $cart
     *
     * @return array
     */
    public function getAddress(ShopgateCart $cart)
    {
        $deliveryAddress = $cart->getDeliveryAddress();
        $address         = empty($deliveryAddress)
            ? $cart->getInvoiceAddress()
            : $deliveryAddress;

        $delivery = array();

        if (!empty($address)) {
            $delivery['zone_id']    =
                $this->getStateId(ShopgateMapper::getShoppingsystemStateCode($address->getState()));
            $delivery['country_id'] =
                $this->getCountryId($address->getCountry());
        }

        return $delivery;
    }

    /**
     * Read the state id from the database by the zone code
     *
     * @param $currentState
     *
     * @return mixed
     */
    private function getStateId($currentState)
    {
        $qry         = "SELECT z.zone_id FROM " . TABLE_ZONES . " AS z where z.zone_code = \"{$currentState}\" ;";
        $result      = ShopgateWrapper::db_query_print_on_err($qry);
        $resultArray = ShopgateWrapper::db_fetch_array($result);

        return $resultArray['zone_id'];
    }

    /**
     * Read the country id from the database by the country iso-2 code
     *
     * @param string $country
     *
     * @return int|null
     */
    private function getCountryId($country)
    {
        return $this->zoneHelper->getCountryByIso2Name($country)->getId();
    }

    /**
     * Generates a token to a customer if no exist
     *
     * Gets token or creates one if needed
     *
     * @param ShopgateCustomer $customer
     *
     * @return null|string
     */
    public function getTokenForCustomer(ShopgateCustomer $customer)
    {
        $token = $this->getToken($customer);
        if (!$token) {
            $id = $this->getIdFromEmail($customer->getMail());
            if ($id) {
                $token = $this->createToken($customer);
                $customer->setCustomerToken($token);
                $this->saveToken($customer);
            }
        }

        return $token;
    }

    /**
     * Get generated token by customer
     *
     * @param ShopgateCustomer $customer
     *
     * @return string|null
     */
    private function getToken(ShopgateCustomer $customer)
    {
        $this->load($customer->getCustomerId());

        return $this->getData('customer_token');
    }

    /**
     * Read the customers database id by his email
     *
     * Simple ID retrieval
     *
     * @param $email - customer's email address
     *
     * @return int|null
     */
    private function getIdFromEmail($email)
    {
        return $this->nativeCustomerModel->load($email, 'customers_email_address')->getCustomersId();
    }

    /**
     * Returns customer token based on
     *
     * @param ShopgateCustomer $customer
     *
     * @return string
     */
    private function createToken(ShopgateCustomer $customer)
    {
        return md5($customer->getMail() . $customer->getCustomerId());
    }

    /**
     * stores a token to an customer in the database
     *
     * @param ShopgateCustomer $customer
     *
     * @return $this
     */
    private function saveToken(ShopgateCustomer $customer)
    {
        $this
            ->setCustomerId($customer->getCustomerId())
            ->setCustomerToken($customer->getCustomerToken());

        return $this->save();
    }

    /**
     * Get the customers database id by the customers token
     *
     * @param string $token
     *
     * @return int|null
     */
    public function getIdFromToken($token)
    {
        $this->load($token, 'customer_token');

        return $this->getId();
    }
}
